import produce from "immer";
import { catchError, from, map, mapTo, merge, of, Subject, take, takeUntil, tap } from "rxjs";
import { ModuleUtils } from "../../../utils/lang";
import { getTypeOperation, retryProcessing } from "../../common/utils";
export const initialStateAsyncCall = {
  params: {},
  status: {},
  errors: []
};
export class AsyncCall {
  constructor(params$, executeFn$, config, destroy$, setState) {
    this.params$ = params$;
    this.executeFn$ = executeFn$;
    this.config = config;
    this.destroy$ = destroy$;
    this.setState = setState;
    this.execute$ = new Subject();
    this.newState = initialStateAsyncCall;
    this.destroy$ = destroy$ ? destroy$ : new Subject();
    this.init = this.init.bind(this);
    this.getState = this.getState.bind(this);
    this.reload = this.reload.bind(this);
  }

  init() {
    const processingStatus$ = new Subject();
    const params$ = this.params$ ? merge(this.params$, this.execute$) : this.execute$;
    const asyncOperation = getTypeOperation(this.config.processingType);
    merge(processingStatus$.pipe(mapTo({
      status: 'PROCESSING'
    })), params$.pipe(takeUntil(this.destroy$), tap(params => {
      this.setState(prevState => {
        const newState = produce(draft => {
          draft.params = params;
        })(prevState);
        this.newState = newState;
        return newState;
      });
      processingStatus$.next();
    }), asyncOperation(params => {
      const data$ = this.executeFn$ ? this.executeFn$(params) : of(params);
      const loadData$ = ModuleUtils.isObservable(data$) ? data$ : ModuleUtils.isPromise(data$) ? from(data$) : of(data$);
      const retryConfig = this.config.retry ? this.config.retry : {
        interval: 100
      };
      return loadData$.pipe(retryProcessing(retryConfig), take(1), map(data => {
        return {
          status: 'SUCCESS',
          data
        };
      }), catchError(error => {
        return of({
          status: 'ERROR',
          error
        });
      }));
    }))).pipe(takeUntil(this.destroy$), tap(processingStatus => {
      const params = this.newState.params;
      processingStatus.status === 'PROCESSING' && this.config.onProcessing && this.config.onProcessing(params);
      processingStatus.status === 'ERROR' && this.config.onError && this.config.onError(processingStatus.error, params);
      processingStatus.status === 'SUCCESS' && this.config.onSuccess && this.config.onSuccess(processingStatus.data, params);
      this.config.onProcessingStatus && this.config.onProcessingStatus(processingStatus, this.newState.params);
    }), tap(processingStatus => {
      this.setState(prevState => {
        const newState = produce(draft => {
          draft.status.status = processingStatus.status;
          draft.status.data = processingStatus.data;
          draft.status.error = processingStatus.error;
        })(prevState);
        this.newState = newState;
        return newState;
      });
    })).subscribe();
  }

  getState() {
    return this.newState;
  }

  reload() {
    this.execute$.next(this.newState.params);
  }

  get execute() {
    return this.execute$.next.bind(this.execute$);
  } // get configx() /*: UseFormGroupConfig*/ {
  //     return {
  //         state: this.newState,
  //         execute: this.execute$.next.bind(this.execute$),
  //         destroy$: this.destroy$,
  //         getData: this.getData
  //     }
  // }


}