"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

exports.__esModule = true;
exports.AsyncCall = exports.initialStateAsyncCall = void 0;

var _immer = _interopRequireDefault(require("immer"));

var _rxjs = require("rxjs");

var _lang = require("../../../utils/lang");

var _utils = require("../../common/utils");

const initialStateAsyncCall = {
  params: {},
  status: {},
  errors: []
};
exports.initialStateAsyncCall = initialStateAsyncCall;

class AsyncCall {
  constructor(params$, executeFn$, config, destroy$, setState) {
    this.params$ = params$;
    this.executeFn$ = executeFn$;
    this.config = config;
    this.destroy$ = destroy$;
    this.setState = setState;
    this.execute$ = new _rxjs.Subject();
    this.newState = initialStateAsyncCall;
    this.destroy$ = destroy$ ? destroy$ : new _rxjs.Subject();
    this.init = this.init.bind(this);
    this.getState = this.getState.bind(this);
    this.reload = this.reload.bind(this);
  }

  init() {
    const processingStatus$ = new _rxjs.Subject();
    const params$ = this.params$ ? (0, _rxjs.merge)(this.params$, this.execute$) : this.execute$;
    const asyncOperation = (0, _utils.getTypeOperation)(this.config.processingType);
    (0, _rxjs.merge)(processingStatus$.pipe((0, _rxjs.mapTo)({
      status: 'PROCESSING'
    })), params$.pipe((0, _rxjs.takeUntil)(this.destroy$), (0, _rxjs.tap)(params => {
      this.setState(prevState => {
        const newState = (0, _immer.default)(draft => {
          draft.params = params;
        })(prevState);
        this.newState = newState;
        return newState;
      });
      processingStatus$.next();
    }), asyncOperation(params => {
      const data$ = this.executeFn$ ? this.executeFn$(params) : (0, _rxjs.of)(params);
      const loadData$ = _lang.ModuleUtils.isObservable(data$) ? data$ : _lang.ModuleUtils.isPromise(data$) ? (0, _rxjs.from)(data$) : (0, _rxjs.of)(data$);
      const retryConfig = this.config.retry ? this.config.retry : {
        interval: 100
      };
      return loadData$.pipe((0, _utils.retryProcessing)(retryConfig), (0, _rxjs.take)(1), (0, _rxjs.map)(data => {
        return {
          status: 'SUCCESS',
          data
        };
      }), (0, _rxjs.catchError)(error => {
        return (0, _rxjs.of)({
          status: 'ERROR',
          error
        });
      }));
    }))).pipe((0, _rxjs.takeUntil)(this.destroy$), (0, _rxjs.tap)(processingStatus => {
      const params = this.newState.params;
      processingStatus.status === 'PROCESSING' && this.config.onProcessing && this.config.onProcessing(params);
      processingStatus.status === 'ERROR' && this.config.onError && this.config.onError(processingStatus.error, params);
      processingStatus.status === 'SUCCESS' && this.config.onSuccess && this.config.onSuccess(processingStatus.data, params);
      this.config.onProcessingStatus && this.config.onProcessingStatus(processingStatus, this.newState.params);
    }), (0, _rxjs.tap)(processingStatus => {
      this.setState(prevState => {
        const newState = (0, _immer.default)(draft => {
          draft.status.status = processingStatus.status;
          draft.status.data = processingStatus.data;
          draft.status.error = processingStatus.error;
        })(prevState);
        this.newState = newState;
        return newState;
      });
    })).subscribe();
  }

  getState() {
    return this.newState;
  }

  reload() {
    this.execute$.next(this.newState.params);
  }

  get execute() {
    return this.execute$.next.bind(this.execute$);
  } // get configx() /*: UseFormGroupConfig*/ {
  //     return {
  //         state: this.newState,
  //         execute: this.execute$.next.bind(this.execute$),
  //         destroy$: this.destroy$,
  //         getData: this.getData
  //     }
  // }


}

exports.AsyncCall = AsyncCall;